//                                               -*- C++ -*-
/**
 *  @file  LinearLeastSquares.hxx
 *  @brief First order polynomial response surface by least square
 *
 *  (C) Copyright 2005-2010 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2010-02-04 16:44:49 +0100 (jeu. 04 févr. 2010) $
 *  Id:      $Id: LinearLeastSquares.hxx 1473 2010-02-04 15:44:49Z dutka $
 */
#ifndef OPENTURNS_LINEARLEASTSQUARES_HXX
#define OPENTURNS_LINEARLEASTSQUARES_HXX

#include "OTprivate.hxx"
#include "PersistentObject.hxx"
#include "NumericalPoint.hxx"
#include "NumericalSample.hxx"
#include "NumericalMathFunction.hxx"
#include "Exception.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Algorithm {

      /**
       * @class LinearLeastSquares
       *
       * The class describes the probabilistic concept of quadratic response surface
       */
      class LinearLeastSquares
	: public Base::Common::PersistentObject
      {
	CLASSNAME;
      public:

	typedef Base::Type::NumericalPoint             NumericalPoint;
	typedef Base::Type::Matrix                     Matrix;
	typedef Base::Func::NumericalMathFunction      NumericalMathFunction;
	typedef Base::Stat::NumericalSample            NumericalSample;
	typedef Base::Common::InvalidArgumentException InvalidArgumentException;

	/** Constructor with parameters */
	explicit LinearLeastSquares(const NumericalSample & dataIn,
				    const NumericalMathFunction & inputFunction,
				    const String & name = OT::DefaultName)
	  /* throw(InvalidArgumentException) */;

	/** Constructor with parameters */
	explicit LinearLeastSquares(const NumericalSample & dataIn,
				    const NumericalSample & dataOut,
				    const String & name = OT::DefaultName)
	  /* throw(InvalidArgumentException) */;


	/** Virtual constructor */
	virtual LinearLeastSquares * clone() const;

	/** String converter */
	String __repr__() const;

	/** Response surface computation */
	void run();

	/** Constant accessor */
	NumericalPoint getConstant() const;

	/** Linear accessor */
	Matrix getLinear() const;

	/** Input function accessor */
	NumericalMathFunction getInputFunction() const;

	/** Response surface accessor */
	NumericalMathFunction getResponseSurface() const;

	/** Data in accessor */
	NumericalSample getDataIn() const;

	/** Data out accessor */
	void setDataOut(const NumericalSample & dataOut)
	  /* throw(InvalidArgumentException) */;
	NumericalSample getDataOut();

      protected:

      private:
	NumericalSample dataIn_;
	NumericalSample dataOut_;
	NumericalMathFunction inputFunction_;
	NumericalMathFunction responseSurface_;
	NumericalPoint constant_;
	Matrix linear_;

      }; /* class LinearLeastSquares */

    } /* namespace Algorithm */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_QUADRATICLEASTSQUARES_HXX */
