//                                               -*- C++ -*-
/**
 *  @file  Normal.hxx
 *  @brief The Normal distribution
 *
 *  (C) Copyright 2005-2010 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2010-02-04 16:44:49 +0100 (jeu. 04 févr. 2010) $
 *  Id:      $Id: Normal.hxx 1473 2010-02-04 15:44:49Z dutka $
 */
#ifndef OPENTURNS_NORMAL_HXX
#define OPENTURNS_NORMAL_HXX

#include "OTprivate.hxx"
#include "EllipticalDistribution.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {

      /**
       * @class Normal
       *
       * The Normal (gaussian) distribution.
       */
      class Normal
        : public Model::EllipticalDistribution
      {
	// Maximum number of samples in the evaluation of the CDF
        static const UnsignedLong MaximumNumberOfPoints; // 10000000
	// Minimum number of samples in the evaluation of the CDF
        static const UnsignedLong MinimumNumberOfPoints; // 1000000
	// Maximum precision required for the CDF in the multidimensional case (dim > 3)
        static const NumericalScalar MaximumCDFEpsilon; // 5.0e-6
	// Minimum precision required for the CDF in the multidimensional case (dim > 3)
        static const NumericalScalar MinimumCDFEpsilon; // 5.0e-2

        CLASSNAME;
      public:

        typedef Model::EllipticalDistribution                                 EllipticalDistribution; // required by SWIG
        typedef EllipticalDistribution::Implementation                        Implementation;
        typedef EllipticalDistribution::InvalidArgumentException              InvalidArgumentException;
        typedef EllipticalDistribution::InvalidDimensionException             InvalidDimensionException;
        typedef EllipticalDistribution::NotDefinedException                   NotDefinedException;
        typedef EllipticalDistribution::NumericalPoint                        NumericalPoint;
        typedef EllipticalDistribution::NumericalSample                       NumericalSample;
        typedef EllipticalDistribution::CovarianceMatrix                      CovarianceMatrix;
        typedef EllipticalDistribution::Indices                               Indices;
        typedef EllipticalDistribution::Interval                              Interval;
        typedef EllipticalDistribution::IsoProbabilisticTransformation        IsoProbabilisticTransformation;
        typedef EllipticalDistribution::InverseIsoProbabilisticTransformation InverseIsoProbabilisticTransformation;
        typedef EllipticalDistribution::CorrelationMatrix                     CorrelationMatrix;
        typedef EllipticalDistribution::SymmetricMatrix                       SymmetricMatrix;
        typedef EllipticalDistribution::SquareMatrix                          SquareMatrix;
        typedef EllipticalDistribution::StorageManager                        StorageManager;

        /** Default constructor */
        explicit Normal(const UnsignedLong dimension = 1)
          /* throw(InvalidArgumentException) */;

        /** Dimension 1 constructor */
        explicit Normal(const NumericalScalar mu, const NumericalScalar sd)
          /* throw(InvalidArgumentException) */;

        /** Constructor for multiD normal distribution */
        explicit Normal(const NumericalPoint & mean,
                        const NumericalPoint & sigma,
                        const CorrelationMatrix & R)
          /* throw(InvalidArgumentException) */;

        explicit Normal(const NumericalPoint & mean,
                        const CovarianceMatrix & C)
          /* throw(InvalidArgumentException) */;


        /** String converter */
	String __repr__() const;

        /* Interface inherited from Distribution */

        /** Virtual constructor */
        Normal * clone() const;

        /** Compute the density generator of the ellipticalal generator, i.e.
         *  the function phi such that the density of the distribution can
         *  be written as p(x) = phi(t(x-mu)R^(-1)(x-mu))                      */
        NumericalScalar computeDensityGenerator(const NumericalScalar betaSquare) const;

        /** Compute the derivative of the density generator */
        NumericalScalar computeDensityGeneratorDerivative(const NumericalScalar betaSquare) const;

        /** Compute the seconde derivative of the density generator */
        NumericalScalar computeDensityGeneratorSecondDerivative(const NumericalScalar betaSquare) const;

        /** Get one realization of the Normal distribution */
        NumericalPoint getRealization() const;

        /** Get the CDF of the Normal distribution */
        using EllipticalDistribution::computeCDF;
        NumericalScalar computeCDF(const NumericalPoint & point, const Bool tail = false) const;

        /** Get the probability content of an interval */
        NumericalScalar computeProbability(const Interval & interval) const;

        /** Get the characteristic function of the distribution, i.e. phi(u) = E(exp(I*u*X)) */
        NumericalComplex computeCharacteristicFunction(const NumericalScalar x,
                                                       const Bool logScale = false) const;

        /** Get the CDF gradient of the distribution */
        NumericalPoint computeCDFGradient(const NumericalPoint & point) const;

        /** Compute the radial distribution CDF */
        NumericalScalar computeRadialDistributionCDF(const NumericalScalar radius) const;

        /** Compute the PDF of Xi | X1, ..., Xi-1. x = Xi, y = (X1,...,Xi-1) */
        NumericalScalar computeConditionalPDF(const NumericalScalar x, const NumericalPoint & y) const;

        /** Compute the CDF of Xi | X1, ..., Xi-1. x = Xi, y = (X1,...,Xi-1) */
        NumericalScalar computeConditionalCDF(const NumericalScalar x, const NumericalPoint & y) const;

        /** Compute the quantile of Xi | X1, ..., Xi-1, i.e. x such that CDF(x|y) = q with x = Xi, y = (X1,...,Xi-1) */
        NumericalScalar computeConditionalQuantile(const NumericalScalar q, const NumericalPoint & y) const;

        /** Get the i-th marginal distribution */
        Implementation getMarginal(const UnsignedLong i) const /* throw(InvalidArgumentException) */;

        /** Get the distribution of the marginal distribution corresponding to indices dimensions */
        Implementation getMarginal(const Indices & indices) const /* throw(InvalidArgumentException) */;

        /** Get the roughness, i.e. the L2-norm of the PDF */
        NumericalScalar getRoughness() const;

        /** Get the skewness of the distribution */
        NumericalPoint getSkewness() const /* throw(NotDefinedException) */;

        /** Get the kurtosis of the distribution */
        NumericalPoint getKurtosis() const /* throw(NotDefinedException) */;

	/** Get the raw moments of the standardized distribution */
	NumericalPoint getStandardMoment(const UnsignedLong n) const;

        /** Sigma vector accessor */
        void setSigma(const NumericalPoint & sigma)
          /* throw(InvalidArgumentException) */;

        /** Correlation matrix accessor */
        void setCorrelation(const CorrelationMatrix & R)
          /* throw(InvalidArgumentException) */;

        /** Get the copula of a distribution */
        Implementation getCopula() const;

        /* Interface specific to Normal */

        /** Tell if the distribution has independent copula */
        Bool hasIndependentCopula() const;

        /** Method save() stores the object through the StorageManager */
        void save(StorageManager::Advocate & adv) const;

        /** Method load() reloads the object from the StorageManager */
        void load(StorageManager::Advocate & adv);

      protected:


      private:

        /** Quantile computation for dimension=1 */
        NumericalScalar computeScalarQuantile(const NumericalScalar prob,
					      const Bool tail = false,
                                              const NumericalScalar precision = DefaultQuantileEpsilon) const;

        /** Check if the distribution has independent copula */
        void checkIndependentCopula();

        /** The normalization factor of the Normal distribution */
        NumericalScalar normalizationFactor_;

        /** Store the independence status */
        Bool hasIndependentCopula_;

      }; /* class Normal */

    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_NORMAL_HXX */
