//                                               -*- C++ -*-
/**
 *  @file  NumericalMathFunctionImplementation.hxx
 *  @brief Abstract top-level class for all numerical math function implementations
 *
 *  (C) Copyright 2005-2010 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2010-02-04 16:44:49 +0100 (jeu. 04 févr. 2010) $
 *  Id:      $Id: NumericalMathFunctionImplementation.hxx 1473 2010-02-04 15:44:49Z dutka $
 */
#ifndef OPENTURNS_NUMERICALMATHFUNCTIONIMPLEMENTATION_HXX
#define OPENTURNS_NUMERICALMATHFUNCTIONIMPLEMENTATION_HXX

#include "PersistentObject.hxx"
#include "NumericalPoint.hxx"
#include "NumericalPointWithDescription.hxx"
#include "NumericalSample.hxx"
#include "Exception.hxx"
#include "Indices.hxx"
#include "Pointer.hxx"
#include "NumericalMathEvaluationImplementation.hxx"
#include "NumericalMathGradientImplementation.hxx"
#include "NumericalMathHessianImplementation.hxx"
#include "Description.hxx"
#include "WrapperFile.hxx"

namespace OpenTURNS {

  namespace Base {

    namespace Func {


      /**
       * @class NumericalMathFunctionImplementation
       *
       * The class that simulates a numerical math function,
       * its gradient and its hessian. This class is just an interface
       * to actual implementation objects that can be hot-replaced
       * during computation. Each implementation object refers to
       * the evaluation, the gradient or the hessian.
       */
      class NumericalMathFunctionImplementation 
	: public Common::PersistentObject
      {
	CLASSNAME;
      public:

	/* Some typedefs for easy reading */
	typedef Common::Pointer<NumericalMathFunctionImplementation>   Implementation;
	typedef Common::Pointer<NumericalMathEvaluationImplementation> EvaluationImplementation;
	typedef Common::Pointer<NumericalMathGradientImplementation>   GradientImplementation;
	typedef Common::Pointer<NumericalMathHessianImplementation>    HessianImplementation;
	typedef Type::NumericalPoint                                   NumericalPoint;
	typedef Type::NumericalPointWithDescription                    NumericalPointWithDescription;
	typedef Stat::NumericalSample                                  NumericalSample;
	typedef Type::Matrix                                           Matrix;
	typedef Type::SymmetricTensor                                  SymmetricTensor;
	typedef Type::Indices                                          Indices;
	typedef Type::Description                                      Description;
	typedef Common::InvalidArgumentException                       InvalidArgumentException;
	typedef Common::InternalException                              InternalException;
	typedef Common::StorageManager                                 StorageManager;

      private:

	/* List of muParser valid constants */
	static Description ValidConstants_;

	/* List of muParser valid functions */
	static Description ValidFunctions_;

	/* List of muParser valid operators */
	static Description ValidOperators_;

	/* Flag to tell if the documentation has been initialized */
	static Bool IsDocumentationInitialized_;

	/* Method that initialize the fields related to the documentation of the analytical functions */
	static void InitializeDocumentation();

      public:

	/** Static methods for documentation of analytical fnctions */
	static Description GetValidConstants();
	static Description GetValidFunctions();
	static Description GetValidOperators();


      public:

	/** Default constructor */
	NumericalMathFunctionImplementation();

	/** Constructor from a wrapper name */
	NumericalMathFunctionImplementation(const String & name);

	/** Constructor from a wrapper file */
	NumericalMathFunctionImplementation(const WrapperFile & wrapperFile);

	/** Analytical formula constructor */
	NumericalMathFunctionImplementation(const Description & inputVariablesNames,
					    const Description & outputVariablesNames,
					    const Description & formulas);

	/** Constructor from implementations */
	NumericalMathFunctionImplementation(const EvaluationImplementation & funcImpl,
					    const GradientImplementation & gradImpl,
					    const HessianImplementation  & hessImpl);

	/** Single function implementation constructor */
	NumericalMathFunctionImplementation(const EvaluationImplementation & evaluationImplementation);

	/** Multiplication of two 1D output functions with the same input dimension */
	virtual NumericalMathFunctionImplementation operator * (const NumericalMathFunctionImplementation & right) const;

	/** Multiplication of two 1D output functions with the same input dimension */
	virtual NumericalMathFunctionImplementation operator * (const Implementation & p_right) const;

	/** Virtual constructor */
	virtual NumericalMathFunctionImplementation * clone() const;

	/** Comparison operator */
	Bool operator ==(const NumericalMathFunctionImplementation & other) const;

	/** String converter */
	virtual String __repr__() const;



	/** Enable or disable the internal cache */
	void enableCache() const;
	void disableCache() const;
	Bool isCacheEnabled() const;


	/* Function implementation accessors */
	void setEvaluationImplementation(const EvaluationImplementation & evaluationImplementation);
	const EvaluationImplementation & getEvaluationImplementation() const;

	/* Gradient implementation accessors */
	void setGradientImplementation(const GradientImplementation & gradientImplementation);
	const GradientImplementation & getGradientImplementation() const;
	
	/* Hessian implementation accessors */
	void setHessianImplementation(const HessianImplementation & hessianImplementation);
	const HessianImplementation & getHessianImplementation() const;

	/* Initial function implementation accessors */
	const EvaluationImplementation & getInitialEvaluationImplementation() const;

	/* Initial gradient implementation accessors */
	const GradientImplementation & getInitialGradientImplementation() const;

	/* Initial hessian implementation accessors */
	const HessianImplementation & getInitialHessianImplementation() const;




	/** Operator () */
	NumericalPoint operator() (const NumericalPoint & in) const
	  /* throw(InvalidArgumentException,InternalException) */;
	NumericalSample operator() (const NumericalSample & in) const
	  /* throw(InvalidArgumentException, InternalException) */;

	/** Method gradient() returns the Jacobian transposed matrix of the function at point */
	Matrix gradient(const NumericalPoint & in) const
	  /* throw(InvalidArgumentException,InternalException) */;

	/** Method hessian() returns the symmetric tensor of the function at point */
	SymmetricTensor hessian(const NumericalPoint & in) const
	  /* throw(InvalidArgumentException,InternalException) */;

	/** Gradient according to the marginal parameters */
	virtual Matrix parametersGradient(const NumericalPoint & in) const;

	/** Parameters value and description accessor */
	virtual NumericalPointWithDescription getParameters() const;
	virtual void setParameters(const NumericalPointWithDescription & parameters);

	/** Accessor for input point dimension */
	UnsignedLong getInputNumericalPointDimension() const
	  /* throw(InternalException) */;

	/** Accessor for output point dimension */
	UnsignedLong getOutputNumericalPointDimension() const
	  /* throw(InternalException) */;

	/** Accessor for input point dimension */
	UnsignedLong getInputDimension() const
	  /* throw(InternalException) */;

	/** Accessor for output point dimension */
	UnsignedLong getOutputDimension() const
	  /* throw(InternalException) */;

	/** Description Accessor, i.e. the names of the input and output parameters */
	void setDescription(const Description & description);
	Description getDescription() const;

	/** Input description Accessor, i.e. the names of the input parameters */
	Description getInputDescription() const;

	/** Output description Accessor, i.e. the names of the Output parameters */
	Description getOutputDescription() const;

	/** Get the i-th marginal function */
	virtual Implementation getMarginal(const UnsignedLong i) const /* throw(InvalidArgumentException) */;

	/** Get the function corresponding to indices components */
	virtual Implementation getMarginal(const Indices & indices) const /* throw(InvalidArgumentException) */;

	/** Number of calls to the evaluation */
	UnsignedLong getEvaluationCallsNumber() const;

	/** Number of calls to the gradient */
	UnsignedLong getGradientCallsNumber() const;

	/** Number of calls to the hessian */
	UnsignedLong getHessianCallsNumber() const;

 	/** Method save() stores the object through the StorageManager */
	void save(StorageManager::Advocate & adv) const;

	/** Method load() reloads the object from the StorageManager */
	void load(StorageManager::Advocate & adv);


      protected:
	/* Initial function implementation accessors */
	void setInitialEvaluationImplementation(const EvaluationImplementation & p_initialEvaluationImplementation);

	/* Initial gradient implementation accessors */
	void setInitialGradientImplementation(const GradientImplementation & p_initialGradientImplementation);

	/* Initial hessian implementation accessors */
	void setInitialHessianImplementation(const HessianImplementation & p_initialHessianImplementation);
    
	/* This method set the implementations with the values listed in the wrapper file */
	void initImplementations(const WrapperFile & wrapperFile);

      private:
	/* A pointer on the actual numerical math function implementation */
	EvaluationImplementation p_evaluationImplementation_;

	/* A pointer on the actual numerical math gradient implementation */
	GradientImplementation p_gradientImplementation_;

	/* A pointer on the actual numerical math hessian implementation */
	HessianImplementation p_hessianImplementation_;

	/* A pointer on the initial actual numerical math function implementation */
	EvaluationImplementation p_initialEvaluationImplementation_;

	/* A pointer on the initial actual numerical math gradient implementation */
	GradientImplementation p_initialGradientImplementation_;

	/* A pointer on the initial actual numerical math hessian implementation */
	HessianImplementation p_initialHessianImplementation_;

	/* Flag to tell if the current gradient is a default implementation */
	mutable Bool useDefaultImplementationGradient_;

	/* Flag to tell if the curren hessian is a default implementation */
	mutable Bool useDefaultImplementationHessian_;

      }; /* class NumericalMathFunctionImplementation */


    } /* namespace Func */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_NUMERICALMATHFUNCTIONIMPLEMENTATION_HXX */
