//                                               -*- C++ -*-
/**
 *  @file  ProjectionStrategyImplementation.hxx
 *  @brief This is a abstract class for projection strategy implementations
 *
 *  (C) Copyright 2005-2010 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-05-21 17:44:02 +0200 (Wed, 21 May 2008) $
 *  Id:      $Id: Object.hxx 818 2008-05-21 15:44:02Z dutka $
 */
#ifndef OPENTURNS_PROJECTIONSTRATEGYIMPLEMENTATION_HXX
#define OPENTURNS_PROJECTIONSTRATEGYIMPLEMENTATION_HXX

#include "PersistentObject.hxx"
#include "NumericalPoint.hxx"
#include "Distribution.hxx"
#include "Collection.hxx"
#include "NumericalMathFunction.hxx"
#include "Indices.hxx"
#include "WeightedExperiment.hxx"
#include "Pointer.hxx"
#include "NumericalSample.hxx"

namespace OpenTURNS
{

  namespace Uncertainty
  {

    namespace Algorithm
    {

      /**
       * @class ProjectionStrategyImplementation
       *
       * This is a abstract class for projection strategy implementations
       */

      class ProjectionStrategyImplementation
        : public Base::Common::PersistentObject
      {
        CLASSNAME;
        // Make the FunctionalChaosAlgorithm class a friend of the ProjectionStrategyImplementation class
        // as this last class is in charge of managing data that are part of the FunctionalChaosAlgorithm
        // state
        friend class FunctionalChaosAlgorithm;

      public:

        typedef Base::Common::StorageManager                  StorageManager;
        typedef Base::Type::NumericalPoint                    NumericalPoint;
	typedef Base::Stat::NumericalSample                   NumericalSample;
        typedef Base::Func::NumericalMathFunction             NumericalMathFunction;
        typedef Base::Type::Collection<NumericalMathFunction> NumericalMathFunctionCollection;
        typedef Model::Distribution                           Distribution;
        typedef Base::Type::Indices                           Indices;
        typedef Base::Common::Pointer<WeightedExperiment>     WeightedExperimentImplementation;

        // friend class Base::Common::Factory<ProjectionStrategyImplementation>;

        /** Default constructor */
        ProjectionStrategyImplementation();

        /** Parameter constructor */
        ProjectionStrategyImplementation(const Distribution & measure);

        /** Parameter constructor */
        ProjectionStrategyImplementation(const WeightedExperiment & weightedExperiment);

        /** Parameter constructor */
        ProjectionStrategyImplementation(const Distribution & measure,
					 const WeightedExperiment & weightedExperiment);

        /** Virtual constructor */
        virtual ProjectionStrategyImplementation * clone() const;

        /** String converter */
        virtual String __repr__() const;

        /** Measure accessor */
        virtual void setMeasure(const Distribution & measure);
        Distribution getMeasure() const;

        /** Experiment accessors */
        virtual void setExperiment(const WeightedExperiment & weightedExperiment);
        virtual WeightedExperiment getExperiment() const;

        /** Compute the components alpha_k_p_ by projecting the model on the partial L2 basis */
        virtual void computeCoefficients(const NumericalMathFunction & function,
                                         const NumericalMathFunctionCollection & partialBasis,
                                         const Indices & indices,
                                         const Indices & addedRanks,
                                         const Indices & conservedRanks,
                                         const Indices & removedRanks);

        /** Method save() stores the object through the StorageManager */
        virtual void save(StorageManager::Advocate & adv) const;

        /** Method load() reloads the object from the StorageManager */
        virtual void load(StorageManager::Advocate & adv);


      protected:

        /** The collection of Alpha_k coefficients */
        NumericalPoint alpha_k_p_;

        /** Residual */
        NumericalScalar residual_p_;

        /** The measureing function for projection */
        Distribution measure_;

        // An  experiment that will be used to discretize the L2 integral
        WeightedExperimentImplementation p_weightedExperiment_;

        // The input sample upon which the leastSquares will be made
        NumericalSample inputSample_;

        // The weights associated with the sample
        NumericalPoint weights_;

        // The sample value of the model
        NumericalSample outputSample_;

      private:

      } ; /* class ProjectionStrategyImplementation */


    } /* namespace Algorithm */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_PROJECTIONSTRATEGYIMPLEMENTATION_HXX */
