//                                               -*- C++ -*-
/**
 *  @file  Solver.hxx
 *  @brief Interface class for a nonlinear scalar solver
 *
 *  (C) Copyright 2005-2010 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2010-02-04 16:44:49 +0100 (jeu. 04 févr. 2010) $
 *  Id:      $Id: Solver.hxx 1473 2010-02-04 15:44:49Z dutka $
 */
#ifndef OPENTURNS_SOLVER_HXX
#define OPENTURNS_SOLVER_HXX

#include "OTprivate.hxx"
#include "SolverImplementation.hxx"
#include "Pointer.hxx"
#include "TypedInterfaceObject.hxx"

namespace OpenTURNS
{

  namespace Base
  {

    namespace Solver
    {

      /**
       * @class Solver
       *
       * This class is an interface for the 1D nonlinear solvers
       */
      class Solver :
        public Common::TypedInterfaceObject<SolverImplementation>
      {
        CLASSNAME;
      public:

        typedef SolverImplementation::NumericalMathFunction     NumericalMathFunction;
        typedef SolverImplementation::InvalidDimensionException InvalidDimensionException;
        typedef SolverImplementation::InternalException         InternalException;
        typedef Common::Pointer<SolverImplementation>           Implementation;

        /** Parameter constructor */
        Solver(const Implementation & p_implementation);

        /** Parameter from an implementation */
        Solver(const SolverImplementation & implementation);

        /** Parameter constructor */
        explicit Solver();

        /** Second parameter constructor */
        Solver(const NumericalScalar absoluteError,
               const NumericalScalar relativeError,
               const UnsignedLong maximumFunctionEvaluation);


        /** Comparison operator */
        Bool operator ==(const Solver & other) const;

        /** String converter */
        virtual String __repr__() const;

        /** Solve attempt to find one root to the equation function(x) = value in [infPoint, supPoint] */
        virtual NumericalScalar solve(const NumericalMathFunction & function,
                                      const NumericalScalar value,
                                      const NumericalScalar infPoint,
                                      const NumericalScalar supPoint)
          /* throw(InternalException, InvalidDimensionException) */;

        /** Solve attempt to find one root to the equation function(x) = value in [infPoint, supPoint] given function(infPoint) and function(supPoint) */
        virtual NumericalScalar solve(const NumericalMathFunction & function,
                                      const NumericalScalar value,
                                      const NumericalScalar infPoint,
                                      const NumericalScalar supPoint,
                                      const NumericalScalar infValue,
                                      const NumericalScalar supValue)
          /* throw(InternalException, InvalidDimensionException) */;

        /** Absolute error accessor */
        void setAbsoluteError(const NumericalScalar absoluteError);
        NumericalScalar getAbsoluteError() const;

        /** Relative error accessor */
        void setRelativeError(const NumericalScalar relativeError);
        NumericalScalar getRelativeError() const;

        /** Maximum function evaluation accessor */
        void setMaximumFunctionEvaluation(const UnsignedLong maximumFunctionEvaluation);
        UnsignedLong getMaximumFunctionEvaluation() const;


      }; /* Class Solver */

    } /* namespace Solver */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_SOLVER_HXX */
