//                                               -*- C++ -*-
/**
 *  @file  SquareMatrix.hxx
 *  @brief SquareMatrix implements the classical mathematical square matrix
 *
 *  (C) Copyright 2005-2010 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2010-02-04 16:44:49 +0100 (jeu. 04 févr. 2010) $
 *  Id:      $Id: SquareMatrix.hxx 1473 2010-02-04 15:44:49Z dutka $
 */
#ifndef OPENTURNS_SQUAREMATRIX_HXX
#define OPENTURNS_SQUAREMATRIX_HXX

#include "OTprivate.hxx"
#include "Matrix.hxx"

namespace OpenTURNS
{

  namespace Base
  {
  
    namespace Type
    {

      class IdentityMatrix;
    
      class SymmetricMatrix;

      /**
       * @class SquareMatrix
       *
       * SquareMatrix implements the classical mathematical square matrix
       */

      class SquareMatrix :
	public Matrix
      {
	CLASSNAME;
	
#ifndef SWIG
	friend SquareMatrix operator * (const NumericalScalar s,
					const SquareMatrix & m);
#endif
	
      public:

	/** Default constructor */
	SquareMatrix();

	/** Constructor with size (dim, which is the same for nbRows_ and nbColumns_) */
	SquareMatrix(const UnsignedLong dim);
#if 0		
	/** Constructor from range of external collection */
	template <class InputIterator>
	SquareMatrix(const UnsignedLong dim,
	             InputIterator first,
		     InputIterator last);
#endif		       
	/** Constructor from external collection */
	/** If the dimensions of the matrix and of the collection */
	/** do not correspond, either the collection is truncated */
	/** or the rest of the matrix is filled with zeros */
	SquareMatrix(const UnsignedLong dim,
	             const Collection<NumericalScalar> & elementsValues);
	
	
	/** String converter */
	String __repr__() const;
	
	/** Get the dimension of the matrix */
	const UnsignedLong getDimension() const;
	
	/** SquareMatrix transpose */
	SquareMatrix transpose () const ;

	/** SquareMatrix additions (must have the same dimensions) */
	SquareMatrix operator + (const SquareMatrix & m) const /* throw(InvalidDimensionException) */;
	SquareMatrix operator + (const SymmetricMatrix & m) const /* throw(InvalidDimensionException) */;
	
	/** SquareMatrix substractions (must have the same dimensions) */
	SquareMatrix operator - (const SquareMatrix & m) const /* throw(InvalidDimensionException) */;
	SquareMatrix operator - (const SymmetricMatrix & m) const /* throw(InvalidDimensionException) */;
	
	/** SquareMatrix multiplications (must have consistent dimensions) */
	SquareMatrix operator * (const SquareMatrix & m) const /* throw(InvalidDimensionException) */;
	SquareMatrix operator * (const SymmetricMatrix & m) const /* throw(InvalidDimensionException) */;
	SquareMatrix operator * (const IdentityMatrix & m) const /* throw(InvalidDimensionException) */;

	/** SquareMatrix integer power */
	SquareMatrix power(const UnsignedLong n) const;

	/** Multiplication with a NumericalPoint (must have consistent dimensions) */
	NumericalPoint operator * (const NumericalPoint & p) const /* throw(InvalidDimensionException) */;

	/** Multiplication with a NumericalScalar */
	SquareMatrix operator * (const NumericalScalar s) const ;
	
	// We import the definitions from the upper class (for Matrix multiplication)
	using Matrix::operator *;
	
	/** Division by a NumericalScalar*/
	SquareMatrix operator / (const NumericalScalar s) const /* throw(InvalidArgumentException) */;
	
	/** Resolution of a linear system */
	NumericalPoint solveLinearSystem(const NumericalPoint & b) const /* throw(InvalidDimensionException) */;
	
	Matrix solveLinearSystem(const Matrix & b) const /* throw(InvalidDimensionException) */;
	
	/** Compute determinant */
	NumericalScalar computeDeterminant() const;
	
	/** Compute eigenvalues */
	NumericalComplexCollection computeEigenValues() const;
	      
	/** Constructor with implementation */
        SquareMatrix(const Implementation & i);

      protected:
      

	
      private:

      }
      
      ; /* class SquareMatrix */


      inline SquareMatrix operator * (const NumericalScalar s,
				      const SquareMatrix & m)
      {
	return m.operator*(s);
      }

	
    } /* namespace Common */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_MATRIX_HXX */
