//                                               -*- C++ -*-
/**
 *  @file  UniVariatePolynomial.hxx
 *  @brief This is a 1D polynomial
 *
 *  (C) Copyright 2005-2010 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-05-21 17:44:02 +0200 (Wed, 21 May 2008) $
 *  Id:      $Id: Object.hxx 818 2008-05-21 15:44:02Z dutka $
 */
#ifndef OPENTURNS_UNIVARIATEPOLYNOMIAL_HXX
#define OPENTURNS_UNIVARIATEPOLYNOMIAL_HXX

#include "OTprivate.hxx"
#include "UniVariatePolynomialImplementation.hxx"
#include "TypedInterfaceObject.hxx"

namespace OpenTURNS
{

  namespace Base
  {

    namespace Func
    {

      /**
       * @class UniVariatePolynomial
       *
       * This is a 1D polynomial
       */

      class UniVariatePolynomial
        : public Common::TypedInterfaceObject<UniVariatePolynomialImplementation>
      {
        CLASSNAME;
      public:

        typedef UniVariatePolynomialImplementation::Implementation             Implementation;
        typedef UniVariatePolynomialImplementation::NumericalPoint             NumericalPoint;
        typedef UniVariatePolynomialImplementation::Matrix                     Matrix;
        typedef UniVariatePolynomialImplementation::NumericalComplexCollection NumericalComplexCollection;
        typedef UniVariatePolynomialImplementation::Coefficients               Coefficients;
        typedef UniVariatePolynomialImplementation::Graph                      Graph;

        /** Default constructor */
        UniVariatePolynomial();

        /** Constructor from coefficients */
        UniVariatePolynomial(const Coefficients & coefficients);

        /** Constructor from implementation */
        UniVariatePolynomial(const UniVariatePolynomialImplementation & implementation);

        /** Constructor from implementation pointer */
        UniVariatePolynomial(const Implementation & p_implementation);

#ifndef SWIG
        /** Constructor from implementation pointer */
        UniVariatePolynomial(UniVariatePolynomialImplementation * p_implementation);
#endif

        /** String converter */
        virtual String __repr__() const;
        virtual String __str__() const;

        /** UniVariatePolynomial are evaluated as functors */
        NumericalScalar operator() (const NumericalScalar x) const;

        /** Multiply the polynomial P by a NumericalScalar */
        UniVariatePolynomial operator * (const NumericalScalar scalar) const;

        /** Multiply the polynomial P by a polynomial Q */
        UniVariatePolynomial operator * (const UniVariatePolynomial & uniVariatePolynomial) const;

        /** Multiply the polynomial by (x to the power deg) */
        UniVariatePolynomial incrementDegree (const UnsignedLong degree = 1) const;

        /** Realize the summation of two polynomials of any degree */
        UniVariatePolynomial operator + (const UniVariatePolynomial & uniVariatePolynomial) const;

        /** Realize the substraction of two polynomials of any degree */
        UniVariatePolynomial operator - (const UniVariatePolynomial & uniVariatePolynomial) const;

        /** Coefficients accessor */
        void setCoefficients(const Coefficients & coefficients);
        Coefficients  getCoefficients() const;

        /** Get the degree of the polynomial */
        UnsignedLong getDegree() const;

        /** Root of the polynomial of degree n as the eigenvalues of the associated matrix */
        NumericalComplexCollection getRoots() const;

        /** Method to draw the graph of the polynomial between given bounds */
        Graph draw(const NumericalScalar xMin,
                   const NumericalScalar xMax,
                   const UnsignedLong pointNumber) const;

      } ; /* class UniVariatePolynomial */


    } /* namespace Func */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_UNIVARIATEPOLYNOMIAL_HXX */
